import os

UWA_ACCOUNT = "wangmo@37.com"
UWA_PASSWORD = "37youxi123456"
ODIN_PROJECT_ID = 1775
TIME_LIMITS = 600
START_APP_TIME = 20
ZG_APP_NAME = 'com.wudi.zgm.uwa'


device_num = "83da43ae"

# 战歌任务按钮、剧情按钮、菜单
ZG_TASK_BUTTON = "txt_downContent"
ZG_PLOT_BUTTON = "imgBottomBg"
ZG_MENU_BUTTON = "But_Menu"


PLATFORM = {"Android": "Android", "iOS": "iOS", "Mac": "MacOS", "Windows": "Windows"}

ROOT_PATH = os.path.dirname(os.path.abspath(__file__))
